@@ -76,28 +76,48 @@ class FlakyPlugin(_FlakyPlugin):
                 self.min_passes,
             )
         original_call_runtest_hook = self.runner.call_runtest_hook
-        self._call_infos = {}
+        self._call_infos[item] = {}
         should_rerun = True
         try:
             self.runner.call_runtest_hook = self.call_runtest_hook
             while should_rerun:
                 self.runner.pytest_runtest_protocol(item, nextitem)
-                call_info = self._call_infos.get(self._PYTEST_WHEN_CALL, None)
+                call_info = self._call_infos.get(item, {}).get(self._PYTEST_WHEN_CALL, None)
                 if call_info is None:
                     return False
-                run = self._call_infos[self._PYTEST_WHEN_CALL]
-                passed = run.excinfo is None
+                passed = call_info.excinfo is None
                 if passed:
                     should_rerun = self.add_success(item)
                 else:
-                    should_rerun = self.add_failure(item, run.excinfo)
+                    should_rerun = self.add_failure(item, call_info.excinfo)
                     if not should_rerun:
-                        item.excinfo = run.excinfo
+                        item.excinfo = call_info.excinfo
         finally:
             self.runner.call_runtest_hook = original_call_runtest_hook
-            del self._call_infos
+            del self._call_infos[item]
         return True
 
+    def _get_test_name_and_err(self, item):
+        """
+        Get the test name and error tuple from a test item.
+
+        :param item:
+            py.test wrapper for the test function to be run
+        :type item:
+            :class:`Function`
+        :return:
+            The test name and error tuple.
+        :rtype:
+            ((`type`, :class:`Exception`, :class:`Traceback`) or (None, None, None), `unicode`)
+        """
+        _, _, name = self._get_test_declaration_callable_and_name(item)
+        call_info = self._call_infos.get(item, {}).get(self._PYTEST_WHEN_CALL, None)
+        if call_info is not None:
+            err = (call_info.excinfo.type, call_info.excinfo.value, call_info.excinfo.tb)
+        else:
+            err = (None, None, None)
+        return err, name
+
     @pytest.hookimpl(tryfirst=True, hookwrapper=True)
     def pytest_runtest_makereport(self, item, call):
         """
@@ -112,7 +132,8 @@ class FlakyPlugin(_FlakyPlugin):
             report.item = item
             report.original_outcome = report.outcome
             if report.failed:
-                if self._will_handle_test_error_or_failure(item, None, None):
+                err, name = self._get_test_name_and_err(item)
+                if self._will_handle_test_error_or_failure(item, name, err):
                     report.outcome = self._PYTEST_OUTCOME_PASSED
 
     @pytest.hookimpl(tryfirst=True, hookwrapper=True)
@@ -131,7 +152,8 @@ class FlakyPlugin(_FlakyPlugin):
                 if self._should_handle_test_success(item):
                     outcome.force_result(self._PYTEST_EMPTY_STATUS)
             elif report.original_outcome == self._PYTEST_OUTCOME_FAILED:
-                if self._will_handle_test_error_or_failure(item, None, None):
+                err, name = self._get_test_name_and_err(item)
+                if self._will_handle_test_error_or_failure(item, name, err):
                     outcome.force_result(self._PYTEST_EMPTY_STATUS)
             delattr(report, 'item')
 
@@ -252,7 +274,7 @@ class FlakyPlugin(_FlakyPlugin):
             lambda: ihook(item=item, **kwds),
             when=when,
         )
-        self._call_infos[when] = call_info
+        self._call_infos[item][when] = call_info
         return call_info
 
     def add_success(self, item):
